// FILENAME:	CTF_Balance_of_Power.cs
//
// AUTHORS:  	Chupie Doll & Youth in Asia
//------------------------------------------------------------------------------

$missionName = "OCC_Alpha_Construct";

exec("multiplayerStdLib.cs");
exec("DMstdLib.cs");

$killPoints 	= 3;
$deathPoints 	= 2;
$occPoints		= 2;		// Points per 30 seconds of occupation
$occKillPoints	= 7;		// Extra Points for killing the occupier (added to $killPoints)
$fortowner = "";

// move the map
$server::HudMapViewOffsetX = -8350;
$server::HudMapViewOffsetY = 150;

function player::onAdd(%this)
{
   say(%this, 0, "Welcome to " @ $missionName @ "!  <F5>NOTE:<F0> Read the <F1>?<F0> tab for rules.");
}

function setDefaultMissionOptions()
{
	$server::TeamPlay = false;
	$server::AllowDeathmatch = false;
	$server::AllowTeamPlay = true;	

   // what can the client choose for a team
	$server::AllowTeamRed = true;
	$server::AllowTeamBlue = true;
	$server::AllowTeamYellow = false;
	$server::AllowTeamPurple = false;

   // what can the server admin choose for available teams
   $server::disableTeamRed = false;
   $server::disableTeamBlue = false;
   $server::disableTeamYellow = true;
   $server::disableTeamPurple = true;
}


function onMissionStart()
{
	initGlobalVars();

	$redBonus = 0;
	$blueBonus = 0;

	$redTK = 0;
	$blueTK = 0;

	$redoccKills = 0;
	$blueoccKills = 0;

	$healRate = 300;   
	$ammoRate = 10;
   	$padWaitTime = 30;
  	$zenWaitTime = 60;

	temperateSounds();
}


function onMissionLoad(){
   cdAudioCycle("Purge", "Terror", "Watching"); 
}

function setRules()
{
      %rules = "<F2>GAME TYPE: \n<F0>Occupation.\n\n"   @        
               "<tIDMULT_TDM_MAPNAME>"    @ 
               $missionName             @
               "\n\n<F2>RULES/OBJECTIVES:\n" @
               "<F0>1) Occupy the Delta Construct and destroy the enemy.\n" @
               "2) Turrets are indestructible: Therefore, destroy the Occupier of the Construct!\n"@
               "3) Only one player can occupy the Construct at a time.\n\n"@
               "<tIDMULT_TDM_SCORING_1>"  @ 
               "<F0>Kills are worth " @ $killPoints @ " points, deaths are worth -" @ $deathPoints @ " points.\n" @
               "Occupiers of the Construct receive " @ $occPoints @ " points every 30 seconds.\n" @
               "Killing the Occupier of the Construct is worth " @ $occKillPoints @ " points.\n\n" @
               "<F2>SERVER RULES/INFORMATION:\n"      @ 
               "<F0>1) All weapons are LEGAL.\n" @
		   "2) Padkilling is LEGAL, particularly if you heal on an enemies pad.\n" @
		   "3) Complaining about items 1 or 2 is ILLEGAL!  Deal with it!  Adapt!  Destroy!\n" @
		   "4) This map was made by Temujin.  You can download it from his website\n" @
 		   "   at http://www6.50megs.com/temujin.  Also at that site is a comprehensive\n" @
		   "   StarSiege training manual and Temujins skin collections.\n\n";

   setGameInfo(%rules);      
}

// setup the rules - this has to be called after the definition of setRules
setRules();

function vehicle::onAdd(%vehicleId)
{
	%player = playerManager::vehicleIdToPlayerNum(%vehicleId);

	%team = getTeam(%vehicleId);
	if(%team == *IDSTR_TEAM_RED) %pcolor = "RED";
	if(%team == *IDSTR_TEAM_BLUE) %pcolor = "BLUE";

	%player.name = getHUDName(%vehicleId);
	if(!%player.occPoints) %player.occPoints = 0;
	if(!%player.occKills) %player.occKills = 0;

	// Ensure that occPoints and occKills stay with the player if they switch teams...

	if(%pcolor != %player.color) {
	   %player.color = %pcolor;
	   if(%pcolor == "RED") {
  		$redBonus += %player.occPoints;
		$blueBonus -= %player.occPoints;
		$red_occKills += %player.occKills;
		$blue_occKills -= %player.occKills;
		}
	   else {
		$redBonus -= %player.occPoints;
		$blueBonus += %player.occPoints;
		$redoccKills -= %player.occKills;
		$blueoccKills += %player.occKills;
	   }
	}
}



// Instant Heal/Reload function
//------------------------------------------------------------------------------

function FastHeal::trigger::onEnter(%this, %object) 
{
      %player = playerManager::vehicleIdToPlayerNum(%object);
      say(%player, 0, "sfx_cybrid_indust.WAV");
	Zen::work(%this, %object, 100000, 100, 0, false);
}



// Fortification Functionality
//------------------------------------------------------------------------------


function Fortify::trigger::OnEnter(%this, %object)
{
	if($fortowner && %object!= $fortowner) {
		say("everybody", 0, getHudName(%object) @ " is ejected from the Alpha Construct!", "sfx_had.WAV");
		rejectvehicle(%object);
		return 0;
		}
	$fortowner = %object;
      say("everybody", 0, getHudName($fortowner)@ " has occupied the Alpha Construct!", "sfx_siren.WAV");
	giveTurretsToThisGuy(%object);
	schedule("raiseBase("@%object@");", 0.25);
}

function Fortify::trigger::OnContact(%this, %object)
{
      %player = playerManager::vehicleIdToPlayerNum(%object);
	%team = getTeam(%object);
	if( %object != $fortowner ) return 0; // shouldn't happen, but just in case
	%player.occPoints = %player.occPoints + $occPoints;
	if(%team == *IDSTR_TEAM_RED) $redBonus = $redBonus + $occPoints;
	if(%team == *IDSTR_TEAM_BLUE) $blueBonus = $blueBonus + $occPoints;
}

function Fortify::trigger::OnLeave(%this, %object)
{
	if( %object == $fortowner ) schedule("neutralizefort();", 2);
}

function vehicle::onDestroyed(%destroyed, %destroyer)
{
   if( %destroyed == $fortowner ) schedule("neutralizefort();", 2);
	
   %team = getTeam(%destroyer);

   if(getHUDName(%destroyer) == "Alpha Construct") {
	if(%team == *IDSTR_TEAM_RED) {
	   $redBonus = $redBonus + 3;
	   $redTK += 1;
	   }
	else if(%team == *IDSTR_TEAM_BLUE) {
	   $blueBonus = $blueBonus + 3;
	   $blueTK += 1;
	   }
   }
   
   if(%destroyed != %destroyer) {
	if(%destroyed == $fortowner) {
         %player = playerManager::vehicleIdToPlayerNum(%destroyer);
   	   %player.occKills += 1;
	   if(%team == *IDSTR_TEAM_RED) $redoccKills += 1;
	   else if(%team == *IDSTR_TEAM_BLUE) $blueoccKills += 1;
	   }
	}

   // Taken from DMStdLib.cs 

   // left over from missionStdLib.cs
   vehicle::onDestroyedLog(%destroyed, %destroyer);
   
   // this is weird but %destroyer isn't necessarily a vehicle
   %message = getFancyDeathMessage(getHUDName(%destroyed), getHUDName(%destroyer));
   if(%message != "")
   {
      say( 0, 0, %message);
   }
   
   // enforce the rules
   if($server::TeamPlay == true)
   {
      if((getTeam(%destroyed) == getTeam(%destroyer)) &&
         (%destroyed != %destroyer) ) {
         antiTeamKill(%destroyer);
      }
   }  
}

function neutralizefort()
{
	if($fortowner == "") return 0;
      say("everybody", 0, "The Alpha Construct is unoccupied!", "sfx_cybrid_alarm.WAV");
	$fortowner = "";
	schedule("makeTurretsNeutral();", 2);
	lowerBase();
}

function raiseBase(%object) 
{
	%target = getObjectId("MissionGroup\\Construct\\Base");

	%count = 21;

   	%startX = getPosition( %object, x );
   	%startY = getPosition( %object, y );

	%diffX = %startX + 8350;		// Actually this is %startX - - 8350...
	%diffY = %startY - 150;

	for( %p = 0; %p < %count; %p++) {
	   %height = 90 + 2.5*%p;
	   %vheight = %height + 85;
	   %angle = 0 - 4.5*%p;
	   %newX = %startX - (%diffX/20)*%p;
	   %newY = %startY - (%diffY/20)*%p;
	   %delay = 0.05*%p;

	   schedule("moveVehicle("@%object@","@%vheight@","@%newX@","@%newY@");", %delay );
	   schedule("moveBase("@%target@","@%height@","@%angle@");", %delay );
    	   }
}

function lowerBase() 
{
	%target = getObjectId("MissionGroup\\Construct\\Base");

	%count = 21;

	for( %p = 0; %p < %count; %p++) {
	   %height = 140 - 2.5*%p;
	   %angle = 0 - 4.5*%p;
	   %delay = 0.05*%p;

	   schedule("moveBase("@%target@","@%height@","@%angle@");", %delay );
    	   }
}

function moveBase(%base, %height, %angle)
{
	setPosition(%base, -8350, 150, %height, %angle);
}

function moveVehicle(%object, %height, %x, %y)
{
	setPosition(%object, %x, %y, %height);
}

function rejectvehicle(%object)
{
      %chx = -8350 - getPosition(%object,x);
      %chy = 150 - getPosition(%object,y);

	%x = -8350 - 2 * %chx;
	%y = 150 - 2 * %chy;
      %z = getPosition(%object,z) + 5;
	
      setPosition(%object, %x, %y, %z);

      %player = playerManager::vehicleIdToPlayerNum(%object);
      say(%player, 0, "sfx_cybrid_alarm.WAV");
}


// Teleport Code
// -----------------------------------------------------------------------------------


function BlueLeap::trigger::OnEnter(%this, %object)
{
	%x = getPosition( %object, x );
	%y = getPosition( %object, y );
	%z = getPosition( %object, z );

	%endX = randomInt( -8380, -8320 );
	%endY = randomInt( -785, -705 );
	%endZ = 280;

	moveIt( %object, %x, %y, %z, %endX, %endY , %endZ);
}

function RedLeap::trigger::OnEnter(%this, %object)
{
	%x = getPosition( %object, x );
	%y = getPosition( %object, y );
	%z = getPosition( %object, z );

	%roll = randomInt( 0, 10 );
	if(%roll == 1) {
	   %endX = randomInt( -8320, -8305 );
	   %endY = randomInt( 1077, 1102 );
	   }
	else if(%roll == 2) {
	   %endX = randomInt( -8395, -8380 );
	   %endY = randomInt( 1077, 1102 );
   	   }
	else {
	   %endX = randomInt( -8395, -8305 );
	   %endY = randomInt( 1015, 1070 );
	   }
	%endZ = 375;

	moveIt( %object, %x, %y, %z, %endX, %endY , %endZ);
}

function moveIt(%object, %sX, %sY, %sZ, %endX, %endY, %endZ)
{
	%x = %sX - %endX;
	%y = %sY - %endY;
	%z = %sZ - %endZ;

      %player = playerManager::vehicleIdToPlayerNum(%object);
      say(%player, 0, "teleporter.WAV");

	%xinc = %x/10;
	%yinc = %y/3;
	%zinc = %z/10;

schedule("subMove("@%object@","@ (%sX - (1*%xinc)) @","@       %sY         @","@ (%sZ - (1*%zinc)) @");", 0.1 );
schedule("subMove("@%object@","@ (%sX - (2*%xinc)) @","@       %sY         @","@ (%sZ - (2*%zinc)) @");", 0.2 );
schedule("subMove("@%object@","@ (%sX - (3*%xinc)) @","@       %sY         @","@ (%sZ - (3*%zinc)) @");", 0.3 );
schedule("subMove("@%object@","@ (%sX - (4*%xinc)) @","@       %sY         @","@ (%sZ - (4*%zinc)) @");", 0.4 );
schedule("subMove("@%object@","@ (%sX - (5*%xinc)) @","@       %sY         @","@ (%sZ - (5*%zinc)) @");", 0.5 );
schedule("subMove("@%object@","@ (%sX - (6*%xinc)) @","@       %sY         @","@ (%sZ - (6*%zinc)) @");", 0.6 );
schedule("subMove("@%object@","@ (%sX - (7*%xinc)) @","@       %sY         @","@ (%sZ - (7*%zinc)) @");", 0.7 );
schedule("subMove("@%object@","@ (%sX - (8*%xinc)) @","@ (%sY - (1*%yinc)) @","@ (%sZ - (8*%zinc)) @");", 0.8 );
schedule("subMove("@%object@","@ (%sX - (9*%xinc)) @","@ (%sY - (2*%yinc)) @","@ (%sZ - (9*%zinc)) @");", 0.9 );
schedule("subMove("@%object@","@   %endX   @", "@   %endY   @", "@   %endZ   @");", 1.0 );
}

function subMove(%object, %x, %y, %z)
{
	setPosition(%object, %x, %y, %z);
}


// Turret Code
// -----------------------------------------------------------------------------------


function giveTurretsToThisGuy(%vehicleId){
	%team = getTeam(%vehicleId);
	setTeam("MissionGroup\\Construct\\T1", %team);
	setTeam("MissionGroup\\Construct\\T2", %team);
	setTeam("MissionGroup\\Construct\\T3", %team);
	setTeam("MissionGroup\\Construct\\T4", %team);
	setTeam("MissionGroup\\Construct\\T5", %team);

	order("MissionGroup\\Construct", Shutdown, false);
}

function makeTurretsNeutral(){
	setTeam("MissionGroup\\Construct\\T1", 0);
	setTeam("MissionGroup\\Construct\\T2", 0);
	setTeam("MissionGroup\\Construct\\T3", 0);
	setTeam("MissionGroup\\Construct\\T4", 0);
	setTeam("MissionGroup\\Construct\\T5", 0);

	order("MissionGroup\\Construct", Shutdown, true);	
}


//------------------------------------------------------------------------------
// scoreboard 



function getPlayerScore(%a)
{
   %p = (getAdjKills(%a)*$killPoints)-(getDeaths(%a)*$deathPoints)+(%a.occKills*$occKillPoints)+%a.occPoints;
   return (%p);
}

function getAdjKills(%a)
{
   return (getKills(%a) - %a.occKills);
}

function getOccPoints(%a)
{
	return(%a.occPoints);
}

function getOccKills(%a)
{
	return(%a.occKills);
}

function getTeamTurretKills(%team)
{
	%teamname = getTeamNameFromTeamId(%team);
	if(%teamname == *IDSTR_TEAM_RED) 
	{
		return($redTK);
	}
	if(%teamname == *IDSTR_TEAM_BLUE) 
	{
		return($blueTK);
	}

	return 0;
}

function getTeamScore(%team)
{
   %teamname = getTeamNameFromTeamId(%team);
   if(%teamname == *IDSTR_TEAM_RED) 
   {
	%p = $redBonus+(getAdjTeamKills(%team)*$killPoints)-(getTeamDeaths(%team)*$deathPoints)+($redoccKills*$occKillPoints);
	return(%p);
   }
   if(%teamname == *IDSTR_TEAM_BLUE) 
   {
	%p = $blueBonus+(getAdjTeamKills(%team)*$killPoints)-(getTeamDeaths(%team)*$deathPoints)+($blueoccKills*$occKillPoints);
	return(%p);
   }

   return 0;
} 

function getAdjTeamKills(%team)
{
   %teamname = getTeamNameFromTeamId(%team);
   if(%teamname == *IDSTR_TEAM_RED) return (getTeamKills(%team) - $redoccKills);
   if(%teamname == *IDSTR_TEAM_BLUE) return (getTeamKills(%team) - $blueoccKills);
   else return 0;
}

function getTeamOccKills(%team)
{
   %teamname = getTeamNameFromTeamId(%team);
   if(%teamname == *IDSTR_TEAM_RED) return $redoccKills;
   if(%teamname == *IDSTR_TEAM_BLUE) return $blueoccKills;
   else return 0;
}

function initScoreBoard()
{
   deleteVariables("$ScoreBoard::PlayerColumn*");
   deleteVariables("$ScoreBoard::TeamColumn*");

   if($server::TeamPlay == "True")	
   {
	   // Player ScoreBoard column headings
	   $ScoreBoard::PlayerColumnHeader1 = *IDMULT_SCORE_TEAM;
	   $ScoreBoard::PlayerColumnHeader2 = *IDMULT_SCORE_SQUAD;
	   $ScoreBoard::PlayerColumnHeader3 = *IDMULT_SCORE_SCORE;
	   $ScoreBoard::PlayerColumnHeader4 = *IDMULT_SCORE_KILLS;
	   $ScoreBoard::PlayerColumnHeader5 = *IDMULT_SCORE_DEATHS;
	   $ScoreBoard::PlayerColumnHeader6 = "Occ. Kills";
	   $ScoreBoard::PlayerColumnHeader7 = "Occ. Points";

	   // Player ScoreBoard column functions
	   $ScoreBoard::PlayerColumnFunction1 = "getTeam";
	   $ScoreBoard::PlayerColumnFunction2 = "getSquad";
	   $ScoreBoard::PlayerColumnFunction3 = "getPlayerScore";
	   $ScoreBoard::PlayerColumnFunction4 = "getKills";
	   $ScoreBoard::PlayerColumnFunction5 = "getDeaths";
	   $ScoreBoard::PlayerColumnFunction6 = "getOccKills";
	   $ScoreBoard::PlayerColumnFunction7 = "getOccPoints";
   }
   else
   {
       // Player ScoreBoard column headings
	   $ScoreBoard::PlayerColumnHeader1 = *IDMULT_SCORE_SQUAD;
	   $ScoreBoard::PlayerColumnHeader2 = *IDMULT_SCORE_SCORE;
	   $ScoreBoard::PlayerColumnHeader3 = *IDMULT_SCORE_KILLS;
	   $ScoreBoard::PlayerColumnHeader4 = *IDMULT_SCORE_DEATHS;

	   // Player ScoreBoard column functions
	   $ScoreBoard::PlayerColumnFunction1 = "getSquad";
	   $ScoreBoard::PlayerColumnFunction2 = "getPlayerScore";
	   $ScoreBoard::PlayerColumnFunction3 = "getKills";
	   $ScoreBoard::PlayerColumnFunction4 = "getDeaths";
   }

   // Team ScoreBoard column headings
   $ScoreBoard::TeamColumnHeader1 = *IDMULT_SCORE_SCORE;
   $ScoreBoard::TeamColumnHeader2 = *IDMULT_SCORE_PLAYERS;
   $ScoreBoard::TeamColumnHeader3 = *IDMULT_SCORE_KILLS;
   $ScoreBoard::TeamColumnHeader4 = *IDMULT_SCORE_DEATHS;
   $ScoreBoard::TeamColumnHeader5 = "Turret Kills";
   $ScoreBoard::TeamColumnHeader6 = "Occ. Kills";

   // Team ScoreBoard column functions
   $ScoreBoard::TeamColumnFunction1 = "getTeamScore";
   $ScoreBoard::TeamColumnFunction2 = "getNumberOfPlayersOnTeam";
   $ScoreBoard::TeamColumnFunction3 = "getAdjTeamKills";
   $ScoreBoard::TeamColumnFunction4 = "getTeamDeaths";
   $ScoreBoard::TeamColumnFunction5 = "getTeamTurretKills";
   $ScoreBoard::TeamColumnFunction6 = "getTeamOccKills";

   // tell server to process all the scoreboard definitions defined above
   serverInitScoreBoard();
}
